"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _weakSet = _interopRequireDefault(require("@babel/runtime/core-js/weak-set"));

var _reactDom = require("react-dom");

var _react = require("react");

var _redboxReact = _interopRequireDefault(require("redbox-react"));

/* eslint-disable no-param-reassign */
var app;
var root;
var currentRouter = null;
var mountedModels = new _weakSet.default();

var hmrRender = function hmrRender(router) {
  app.router(router);
  (0, _reactDom.render)((0, _react.createElement)(app.start()), root);
};

var hot = {
  patch: function patch(inst, container) {
    if (app) {
      console.error("[hot.patch] You have been patch app, don't patch app twice");
      return inst;
    }

    if (!isDvaInstance(inst)) {
      console.error('[hot.patch] app should be a `dva` instance');
      return inst;
    }

    app = inst;

    if (container !== undefined) {
      hot.setContainer(container);
    }

    patchAppStart(inst);
    patchAppModel(inst);
    patchAppRouter(inst);
    console.log('[dva-hot] enabled.');
    return inst;
  },
  setContainer: function setContainer(container) {
    container = normalizeContainer(container);

    if (!container) {
      console.error('container should be selector or Element');
    } else {
      root = container;
    }

    return container;
  },
  model: function model(sourceModule) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.model] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (model) {
        if (!isDvaModel(model)) {
          throw Error('[hot.model] model shoule be a obj and has `namespace` prop');
        }

        sourceModule.hot.accept();
        sourceModule.hot.dispose(function (data) {
          if (mountedModels.has(model)) {
            data.namespace = model.namespace;
            mountedModels.delete(model);
            model = undefined;
            data.enabled = true;
          }
        });

        if (shouldReplaceModule(sourceModule)) {
          var eventL = function eventL(ev) {
            ev.preventDefault();
            renderException(ev.error);
          };

          window.addEventListener('error', eventL);

          try {
            app.unmodel(sourceModule.hot.data.namespace);
            app.model(model);
          } catch (e) {
            console.error('error', e);
          }

          window.removeEventListener('error', eventL);
        }

        return model;
      };
    }

    return passthrough;
  },
  router: function router(sourceModule) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.router] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (router) {
        if (!isFunction(router)) {
          throw Error('[hot.router] router should be a function');
        }

        sourceModule.hot.accept();
        sourceModule.hot.dispose(function (data) {
          data.enabled = currentRouter === router;
          router = undefined;
        });

        if (shouldReplaceModule(sourceModule)) {
          replaceRouter(router);
        }

        return router;
      };
    }

    return passthrough;
  }
};
var _default = hot;
exports.default = _default;

function isHTMLElement(node) {
  return (0, _typeof2.default)(node) === 'object' && node !== null && node.nodeType && node.nodeName;
}

function isString(str) {
  return typeof str === 'string';
}

function isFunction(func) {
  return typeof func === 'function';
}

function isDvaInstance(inst) {
  return inst && isFunction(inst.start) && isFunction(inst.model) && isFunction(inst.router) && isFunction(inst.use);
}

function isDvaModel(obj) {
  return obj && obj.namespace;
}

function isObject(obj) {
  return obj && (0, _typeof2.default)(obj) === 'object';
}

function passthrough(x) {
  return x;
}

function isNodeModule(x) {
  return isObject(x) && 'id' in x;
}

function renderException(error) {
  (0, _reactDom.render)((0, _react.createElement)(_redboxReact.default, {
    error: error
  }), root);
}

function replaceRouter(router) {
  currentRouter = router;

  try {
    hmrRender(router);
  } catch (error) {
    console.error('error', error);
    renderException(error);
  }
}

function normalizeContainer(container) {
  if (isString(container)) {
    container = document.querySelector(container);
  }

  if (isHTMLElement(container)) {
    return container;
  }

  return null;
}

function shouldReplaceModule(sourceModule) {
  return app && root && sourceModule.hot.data && sourceModule.hot.data.enabled;
}

function patchAppStart(inst) {
  var oldStart = inst.start;

  inst.start = function (container) {
    if (container !== undefined) {
      container = hot.setContainer(container);
    }

    if (container) {
      app.use({
        onHmr: function onHmr(r) {
          hmrRender = r;
        }
      });
    }

    return oldStart.call(inst, container);
  };
}

function patchAppModel(inst) {
  var oldMethod = inst.model;

  inst.model = function (m) {
    if (isDvaModel(m)) {
      mountedModels.add(m);
    }

    return oldMethod.call(inst, m);
  };
}

function patchAppRouter(inst) {
  var oldMethod = inst.router;

  inst.router = function (r) {
    if (isFunction(r)) {
      currentRouter = r;
    }

    return oldMethod.call(inst, r);
  };
}